/*
 * Project name:
     Hall current click (Using mikroE's Current click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release (FJ);
 * Description:
     This project is a simple demonstration of working with the Hall Current Click board
     based on the TLI4970 high precision miniature coreless magnetic current sensorwith SPI Interface.
     This example demosntrates measuring the current drawn by the consumer.
 * Test configuration:
     MCU:             P32MX795F512L
                      http://ww1.microchip.com/downloads/en/DeviceDoc/61156F.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     ext. modules:    hall current click board click
                      http://www.mikroe.com/click/hall-current/
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 * NOTES:
     - Place Hall current click board in the mikroBUS socket 1.
     - Turn on TFT backlight on SW11.1.
     - Maximal SPI clock frequency for TLI4970 running at 3.3V is 5MHz.
 */

#include "resources.h"

sbit Chip_Select           at RC2_bit;
sbit Chip_Select_Direction at TRISC2_bit;

char txt[15];
int current_read;
float current_read_float;

// Get current value
unsigned Hall_Read() {
  // Local variable
  unsigned int tmp = 0;
  unsigned int buffer = 0;

  Chip_Select = 0;          // Select MCP3201
  tmp = SPI3_Read(buffer);  // Get value
  Chip_Select = 1;          // Deselect MCP3201

  return tmp;               // Returns value
}

void DrawFrame() {
  TFT_Init_ILI9341_8bit(320,240);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("Hall  Click  Board  Demo", 55, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyPIC Fusion v7", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}

void main() {
  // Draw Graphical Frame
  DrawFrame();

  // Set chip select pin to be output
  Chip_Select_Direction = 0;
  Chip_Select = 1;                                              // Deselect MCP3201

  // Initialize SPI3 module
  SPI3_Init_Advanced(_SPI_MASTER, _SPI_16_BIT, 64, _SPI_SS_ENABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_ACTIVE_2_IDLE);
  Delay_ms(100);

  // Display string on the TFT
  TFT_Write_Text("Current [A] :", 85, 110);

  while(1) {
    // Read current from Hall Click
    current_read = Hall_Read();
    
    // If Sensor Current Value Transmission
    if (current_read.B15 == 0) {
      current_read = current_read & 0x1FFF;                     // Get [13:0] bits of current value
      current_read = current_read - 4096;                       // Map the current value
      current_read_float = (float)current_read * 0.0125;        // Calculate the current value in Amperes
      FloatToStr(current_read_float,txt);                       // Convert the current value to a string
      Ltrim(txt);                                               // Trim the string from leading spaces

      // Display the current value
      TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
      TFT_Write_Text(txt, 170, 110);
      delay_ms(500);
      TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
      TFT_Write_Text(txt, 170, 110);
    }
  }
}